`define NUM_CORE 2

module oc_fpu100_top(handsome_clk, handsome_din_top, handsome_dout_top, clk_i);
	input handsome_clk;
	input handsome_din_top;
	output handsome_dout_top;
	input clk_i;

localparam input_width = 70;
localparam output_width = 41;
localparam reducer_log4 = 3;
localparam reducer_log4_top = 4;

wire [`NUM_CORE-1:0] start_i;
wire [`NUM_CORE-1:0] ready_o;
wire [`NUM_CORE-1:0] ine_o;
wire [`NUM_CORE-1:0] overflow_o;
wire [`NUM_CORE-1:0] underflow_o;
wire [`NUM_CORE-1:0] div_zero_o;
wire [`NUM_CORE-1:0] inf_o;
wire [`NUM_CORE-1:0] zero_o;
wire [`NUM_CORE-1:0] qnan_o;
wire [`NUM_CORE-1:0] snan_o;
wire [31:0] opa_i [`NUM_CORE-1:0];
wire [31:0] opb_i [`NUM_CORE-1:0];
wire [2:0] fpu_op_i [`NUM_CORE-1:0];
wire [1:0] rmode_i [`NUM_CORE-1:0];
wire [31:0] output_o [`NUM_CORE-1:0];
wire [`NUM_CORE-1:0] handsome_din;
wire [`NUM_CORE-1:0] handsome_dout;

handsome_lfsr #(`NUM_CORE) input_lfsr_top (
			.clock(handsome_clk),
			.input_bit(handsome_din_top),
			.output_vector(handsome_din));

generate
begin:G1
	genvar i;
	for (i = 0; i < `NUM_CORE; i = i+1)
	begin :stamp_inst

		handsome_lfsr #(input_width) input_lfsr (
			.clock(handsome_clk),
			.input_bit(handsome_din[i]),
			.output_vector({start_i[i],opa_i[i],opb_i[i],fpu_op_i[i],rmode_i[i]}));

		oc_fpu100 oc_stamp(
			.clk_i(clk_i),
			.start_i(start_i[i]),
			.opa_i(opa_i[i]),
			.opb_i(opb_i[i]),
			.fpu_op_i(fpu_op_i[i]),
			.rmode_i(rmode_i[i]),
			.ready_o(ready_o[i]),
			.ine_o(ine_o[i]),
			.overflow_o(overflow_o[i]),
			.underflow_o(underflow_o[i]),
			.div_zero_o(div_zero_o[i]),
			.inf_o(inf_o[i]),
			.zero_o(zero_o[i]),
			.qnan_o(qnan_o[i]),
			.snan_o(snan_o[i]),
			.output_o(output_o[i]));

		handsome_reducer #(reducer_log4) output_reducer (
			.clock(handsome_clk),
			.input_vector({ready_o[i],ine_o[i],overflow_o[i],underflow_o[i],div_zero_o[i],inf_o[i],zero_o[i],qnan_o[i],snan_o[i],output_o[i]}),
			.output_bit(handsome_dout[i]));

	end

handsome_reducer #(reducer_log4_top) output_reducer_top (
	.clock(handsome_clk),
	.input_vector(handsome_dout[`NUM_CORE-1:0]),
	.output_bit(handsome_dout_top));

end
endgenerate
endmodule
